include 'mkl/mkl_pardiso.f90'

module sparse_solve

use MKL_PARDISO

implicit none

contains

! Solving sparse system of linear equations
function sparse(A0,b0)
! --- Declaration
! In
real(8), intent(in) :: A0(:,:),b0(:)
! Out
real(8) :: sparse(size(b0))
! Local
integer :: count,count1,i,j,take,maxfct,mnum,mtype,phase,n,nrhs,iparm(64),msglvl,error
integer, allocatable :: ia(:),ja(:),perm(:)
real(8), allocatable :: a(:),b(:),x(:)
type(MKL_PARDISO_HANDLE) :: pt(64)
! --- Body
maxfct=1
mnum=1
mtype=11
n=size(b0)
nrhs=1
iparm(1)=0
msglvl=0
! Determine number of non-zero elements
count=0
do j=1,n
    do i=1,n
        if (A0(i,j)/=0d0) then
            count=count+1
        end if
    end do
end do
write(*,'(1X,A,I3,A,I4)'), 'Number of non-zero elements: ', count, ' out of ', n**2
! Store sparse matrix in CSR format
allocate(a(count),ia(n+1),ja(count),perm(n),b(n),x(n))
b=b0
count=0
count1=0
do i=1,n
    take=1
    do j=1,n
        if (A0(i,j)/=0d0) then
            count=count+1
            a(count)=A0(i,j)
            ja(count)=j
            if (take==1) then
                count1=count1+1
                ia(count1)=count
                take=0
            end if
        end if
    end do
end do
ia(n+1)=count+1
! Pointer for pardiso
do i=1,64
   pt(i)%dummy=0 
end do
! Call pardiso
phase=13
call pardiso(pt,maxfct,mnum,mtype,phase,n,a,ia,ja,perm,nrhs,iparm,msglvl,b,x,error)
print*, 'error = ', error
sparse=x
deallocate(a,ia,ja,perm,b,x)
end function sparse

end module sparse_solve